#!/bin/sh

DATA=data
CAT=$1

if [ -z "$CAT" ]; then
    BENCHMARKS=$(cat benchmarks/list.txt)
else
    BENCHMARKS=$(cat benchmarks/list-$CAT.txt)
fi

ALLTOOLS=`cat <<END
L2Sia-wfr
L2Sia
HSF
Ultimate-LTL
T2-ctlstar
T2-termination
END
`

TOOLS=`for t in $ALLTOOLS; do ls data | grep '^'$t'$'; done`

##########
# PROLOG #
##########

echo "<html>"
echo "<head>"
echo "<style>"
echo "table {border-collapse: collapse;} th {text-align: left;} th, td {padding: 0 8px;} .safe {font-weight: bold; color: darkgreen;} .unsafe {font-weight: bold; color: red;} .wrong {font-weight: bold; color: gray;} tr:hover {background-color: lightgray;} .separate {border-left: 1px solid black;}"
echo "</style>"
echo "</head>"
echo "<body>"
echo "<table>"

##############
# TABLE HEAD #
##############

echo -n "<tr><th></th>"
TOOLNUM=0
for t in $TOOLS; do
    eval   SAFE_$TOOLNUM=0
    eval UNSAFE_$TOOLNUM=0
    eval   TIME_$TOOLNUM=0
    eval   GAIN_$TOOLNUM=0
    eval   LOSS_$TOOLNUM=0

    if [ $TOOLNUM -gt 0 ]; then
        SPAN=7
    else
        SPAN=4
    fi

    echo -n "<th class=\"separate\" colspan=\"$SPAN\" style=\"text-align: center;\">$t</th>"

    TOOLNUM=`expr $TOOLNUM + 1`
done
echo "</tr>"
echo -n "<tr><th></th>"
TOOLNUM=0
for t in $TOOLS; do
    echo -n "<th colspan=\"3\" class=\"separate\">result</th><th>time</th>"
    if [ $TOOLNUM -gt 0 ]; then
        echo -n "<th colspan=\"3\">delta</th>"
    fi

    TOOLNUM=`expr $TOOLNUM + 1`
done
echo "</tr>"

##############
# BENCHMARKS #
##############

for b in $BENCHMARKS; do
    B=$(echo $b | sed 's:[^/]*/::')
    echo -n "<tr><th>$B</th>"
    TOOLNUM=0

    ###################
    # COLUMN PER TOOL #
    ###################

    for t in $TOOLS; do
        FILE=$DATA/$t/$B
        RESULT=`cat $FILE 2>/dev/null | cut -d" " -f 2`
        TIME=`cat $FILE 2>/dev/null | cut -d" " -f 4`
        STYLE=""
        INCLUDE=true

        if [ $TOOLNUM -eq 0 ]; then
            BASE_RESULT=$RESULT
        fi
        if [ x$RESULT = xsafe ]; then
            STYLE=" safe"
        fi
        if [ x$RESULT = xunsafe ]; then
            STYLE=" unsafe"
        fi
        if [ ! -f $FILE ]; then
            RESULT="-"
            TIME="-"
        fi
        if [ $TOOLNUM -gt 0 ]; then
            if [ -f $FILE ]; then
                if [ x$BASE_RESULT = 'x?' ] && [ x$RESULT != 'x?' ] && [ -n "$RESULT" ]; then
                    eval GAIN_$TOOLNUM=`eval expr '$GAIN_'$TOOLNUM + 1`
                else
                    if [ x$BASE_RESULT != 'x?' ] && [ x$RESULT = 'x?' ] && [ -n "$BASE_RESULT" ]; then
                        eval LOSS_$TOOLNUM=`eval expr '$LOSS_'$TOOLNUM + 1`
                    else
                        if [ x$BASE_RESULT != x$RESULT ]; then
                            eval LOSS_$TOOLNUM=`eval expr '$LOSS_'$TOOLNUM + 1`
                            INCLUDE=false
                            STYLE=" wrong"
                        fi
                    fi
                fi
            fi
        fi
        echo -n "<td colspan=\"3\" class=\"separate$STYLE\">$RESULT</td><td>$TIME</td>"
        if [ $TOOLNUM -gt 0 ]; then
            echo -n "<td colspan=\"3\">"
            if [ ! -f $FILE ]; then
                echo -n "-"
            else
                if [ x$BASE_RESULT = 'x?' ] && [ x$RESULT != 'x?' ] && [ -n "$RESULT" ]; then
                    echo -n "+1"; # Solved more than baseline tool could
                else
                    if [ x$BASE_RESULT != 'x?' ] && [ x$RESULT = 'x?' ] && [ -n "$BASE_RESULT" ]; then
                        echo -n "-1"; # Couldnt solve what baseline tool could
                    else
                        if [ x$BASE_RESULT = x$RESULT ]; then
                            echo -n "0" # Same result
                        else
                            echo -n "?" # Different results
                        fi
                    fi
                fi
            fi
            echo -n "</td>"
        fi
        if $INCLUDE && [ x$RESULT = xsafe ]; then
            eval SAFE_$TOOLNUM=`eval expr '$SAFE_'$TOOLNUM + 1`
            eval TIME_$TOOLNUM=`eval echo '$TIME_'$TOOLNUM + $TIME | bc -l`
        fi
        if $INCLUDE && [ x$RESULT = xunsafe ]; then
            eval UNSAFE_$TOOLNUM=`eval expr '$UNSAFE_'$TOOLNUM + 1`
            eval   TIME_$TOOLNUM=`eval echo '$TIME_'$TOOLNUM + $TIME | bc -l`
        fi
        TOOLNUM=`expr $TOOLNUM + 1`
    done
    echo "</tr>"
done

#############################
# TABLE FOOT WITH SUMMARIES #
#############################

echo -n "<tr style=\"background-color: lightgray;\"><th rowspan=\"2\"># Benchmarks: `wc -l <<END
$BENCHMARKS
END`</th>"
TOOLNUM=0
for t in $TOOLS; do
    echo -n "<th class=\"separate\">total</th><th>safe</th><th>unsafe</th><th>time</th>"
    if [ $TOOLNUM -gt 0 ]; then
        echo -n "<th>delta</th><th>gained</th><th>lost</th>";
    fi
    TOOLNUM=`expr $TOOLNUM + 1`
done
echo "</tr>"
echo -n "<tr style=\"background-color: lightgray;\">"
TOOLNUM=0
for t in $TOOLS; do
    echo -n "<td class=\"separate\">`eval expr '$SAFE_'$TOOLNUM + '$UNSAFE_'$TOOLNUM`</td><td>`eval echo '$SAFE_'$TOOLNUM`</td><td>`eval echo '$UNSAFE_'$TOOLNUM`</td><td>`eval echo '$TIME_'$TOOLNUM`s</td>"
    if [ $TOOLNUM -gt 0 ]; then
        echo -n "<td>`eval expr '$GAIN_'$TOOLNUM - '$LOSS_'$TOOLNUM`</td><td>`eval echo '$GAIN_'$TOOLNUM`</td><td>`eval echo '$LOSS_'$TOOLNUM`</td>";
    fi
    TOOLNUM=`expr $TOOLNUM + 1`
done
echo "</tr>"

##########
# EPILOG #
##########

echo "</table>"
echo "</body>"
echo "</html>"
